@@ -1,5 +1,5 @@
 
-__author__    = "Andre Merzky, Mark Santcroos, Ole Weidner"
+__author__    = "Andre Merzky, Ole Weidner"
 __copyright__ = "Copyright 2012-2013, The SAGA Project"
 __license__   = "MIT"
 
@@ -7,10 +7,9 @@ __license__   = "MIT"
 """ PBS job adaptor implementation
 """
 
-import saga.utils.which
-import saga.utils.pty_shell
-import saga.utils.threads   as sut
+import threading
 
+import saga.utils.pty_shell
 import saga.adaptors.base
 import saga.adaptors.cpi.job
 
@@ -24,11 +23,11 @@ import threading
 from copy import deepcopy
 from cgi  import parse_qs
 
-SYNC_CALL = saga.adaptors.cpi.decorators.SYNC_CALL
+SYNC_CALL  = saga.adaptors.cpi.decorators.SYNC_CALL
 ASYNC_CALL = saga.adaptors.cpi.decorators.ASYNC_CALL
 
 SYNC_WAIT_UPDATE_INTERVAL = 1  # seconds
-MONITOR_UPDATE_INTERVAL = 3  # seconds
+MONITOR_UPDATE_INTERVAL   = 3  # seconds
 
 
 # --------------------------------------------------------------------
@@ -40,7 +39,7 @@ class _job_state_monitor(threading.Thread):
 
         self.logger = job_service._logger
         self.js = job_service
-        self._stop = sut.Event()
+        self._stop = threading.Event()
 
         super(_job_state_monitor, self).__init__()
         self.setDaemon(True)
@@ -163,8 +162,9 @@ def _pbscript_generator(url, logger, jd, ppn, pbs_version, is_cray=False, queue=
 
     # a workaround is to do an explicit 'cd'
     if jd.working_directory is not None:
-        workdir_directives  = 'export PBS_O_WORKDIR=%s \n' % jd.working_directory
-        workdir_directives += 'cd $PBS_O_WORKDIR \n'
+        workdir_directives  = 'export    PBS_O_WORKDIR=%s \n' % jd.working_directory
+        workdir_directives += 'mkdir -p  %s\n' % jd.working_directory
+        workdir_directives += 'cd        %s\n' % jd.working_directory
     else:
         workdir_directives = ''
 
@@ -355,11 +355,10 @@ class Adaptor (saga.adaptors.base.Base):
     #
     def __init__(self):
 
-        saga.adaptors.base.Base.__init__(self,
-            _ADAPTOR_INFO, _ADAPTOR_OPTIONS)
+        saga.adaptors.base.Base.__init__(self, _ADAPTOR_INFO, _ADAPTOR_OPTIONS)
 
         self.id_re = re.compile('^\[(.*)\]-\[(.*?)\]$')
-        self.opts = self.get_config()
+        self.opts  = self.get_config (_ADAPTOR_NAME)
 
     # ----------------------------------------------------------------
     #
@@ -558,6 +557,10 @@ class PBSJobService (saga.adaptors.cpi.job.Service):
         # get the job description
         jd = job_obj.jd
 
+        # normalize working directory path
+        if  jd.working_directory :
+            jd.working_directory = os.path.normpath (jd.working_directory)
+
         if (self.queue is not None) and (jd.queue is not None):
             self._logger.warning("Job service was instantiated explicitly with \
 'queue=%s', but job description tries to a differnt queue: '%s'. Using '%s'." %
@@ -590,7 +593,7 @@ class PBSJobService (saga.adaptors.cpi.job.Service):
         # (1) we create a temporary file with 'mktemp' and write the contents of 
         #     the generated PBS script into it
         # (2) we call 'qsub <tmpfile>' to submit the script to the queueing system
-        cmdline = """SCRIPTFILE=`mktemp -t SAGA-Python-PBSJobScript.XXXXXX` &&  echo "%s" > $SCRIPTFILE && %s $SCRIPTFILE""" %  (script, self._commands['qsub']['path'])
+        cmdline = """SCRIPTFILE=`mktemp -t SAGA-Python-PBSJobScript.XXXXXX` &&  echo "%s" > $SCRIPTFILE && %s $SCRIPTFILE && rm -f $SCRIPTFILE""" %  (script, self._commands['qsub']['path'])
         ret, out, _ = self.shell.run_sync(cmdline)
 
         if ret != 0:
