@@ -258,7 +258,8 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self.cwdurl      = saga.Url (url) # deep copy
         self.cwdurl.path = self.cwd
 
-        self.shell = sups.PTYShell     (self.url, self.session, self._logger)
+        self.shell       = sups.PTYShell     (self.url, self.session, self._logger)
+        self._copy_shell = None
 
       # self.shell.set_initialize_hook (self.initialize)
       # self.shell.set_finalize_hook   (self.finalize)
@@ -274,6 +275,18 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
 
         return self.get_api ()
 
+
+    # ----------------------------------------------------------------
+    #
+    def _get_copy_shell (self, tgt) :
+
+        if  not self._copy_shell :
+            self._logger.debug ("new copy shell (%s)" % (tgt))
+            self._copy_shell = sups.PTYShell (tgt, self.session, self._logger)
+
+        return self._copy_shell
+
+
     # ----------------------------------------------------------------
     #
     def initialize (self) :
@@ -353,6 +366,42 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         return saga.filesystem.Directory (url=url, flags=flags, session=self.session, 
                                           _adaptor=self._adaptor, _adaptor_state=adaptor_state)
 
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def change_dir (self, tgt, flags) :
+
+        cwdurl = saga.Url (self.url)
+        tgturl = saga.Url (tgt)
+
+        if  not sumisc.url_is_compatible (cwdurl, tgturl) :
+            raise saga.BadParameter ("target dir outside of namespace '%s': %s" \
+                                  % (cwdurl, tgturl))
+
+        cmd = None
+
+        if  tgturl.path == '.' or \
+            tgturl.path == cwdurl.path :
+            self._logger.debug ("change directory optimized away (%s) == (%s)" % (cwdurl, tgturl))
+
+
+        if  flags & saga.filesystem.CREATE_PARENTS :
+            cmd = "mkdir -p %s ;  cd %s" % (tgturl.path, tgturl.path)
+        elif flags & saga.filesystem.CREATE :
+            cmd = "mkdir    %s ;  cd %s" % (tgturl.path, tgturl.path)
+        else :
+            cmd = "test -d  %s && cd %s" % (tgturl.path, tgturl.path)
+
+        ret, out, _ = self.shell.run_sync (cmd)
+
+        if  ret != 0 :
+            raise saga.BadParameter ("invalid dir '%s': %s" % (cwdurl, tgturl))
+
+        self._logger.debug ("changed directory (%s)(%s)" % (ret, out))
+
+        self.valid = True
+
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
@@ -429,8 +478,8 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         src    = saga.Url (src_in)   # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-      # if  sumisc.url_is_relative (src) : src = sumisc.url_make_absolute (cwdurl, src)
-      # if  sumisc.url_is_relative (tgt) : tgt = sumisc.url_make_absolute (cwdurl, tgt)
+        if  sumisc.url_is_relative (src) : src = sumisc.url_make_absolute (cwdurl, src)
+        if  sumisc.url_is_relative (tgt) : tgt = sumisc.url_make_absolute (cwdurl, tgt)
     
         rec_flag = ""
         if  flags & saga.filesystem.RECURSIVE : 
@@ -462,17 +511,16 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
             if  not sumisc.url_is_local (cwdurl) :
 
                 # print "cwd remote"
-
                 if  sumisc.url_is_local (src)          and \
                     sumisc.url_is_compatible (cwdurl, tgt) :
 
-                    # print "from local to remote"
+                  # print "from local to remote: %s -> %s" % (src.path, tgt.path)
                     files_copied = self.shell.stage_to_remote (src.path, tgt.path, rec_flag)
 
                 elif sumisc.url_is_local (tgt)          and \
                      sumisc.url_is_compatible (cwdurl, src) :
 
-                    # print "from remote to loca"
+                  # print "from remote to local: %s -> %s" % (src.path, tgt.path)
                     files_copied = self.shell.stage_from_remote (src.path, tgt.path, rec_flag)
 
                 else :
@@ -497,8 +545,8 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
                                               % (tgt))
 
                     # print "from local to remote"
-                    tmp_shell    = sups.PTYShell (tgt, self.session, self._logger)
-                    files_copied = tmp_shell.stage_to_remote (src.path, tgt.path, rec_flag)
+                    copy_shell   = self._get_copy_shell (tgt)
+                    files_copied = copy_shell.stage_to_remote (src.path, tgt.path, rec_flag)
 
                 elif sumisc.url_is_local (tgt) :
 
@@ -508,8 +556,8 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
                                               % (src))
 
                     # print "from remote to local"
-                    tmp_shell    = sups.PTYShell (src, self.session, self._logger)
-                    files_copied = tmp_shell.stage_from_remote (src.path, tgt.path, rec_flag)
+                    copy_shell   = self._get_copy_shell (tgt)
+                    files_copied = copy_shell.stage_from_remote (src.path, tgt.path, rec_flag)
 
                 else :
 
@@ -523,6 +571,56 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
             _from_task._set_metric ('files_copied', files_copied)
 
 
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def link_self (self, tgt, flags):
+
+        self._is_valid ()
+
+        # FIXME: eval flags
+
+        return self.link (self.url, tgt, flags)
+   
+   
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def link (self, src_in, tgt_in, flags, _from_task=None):
+
+        # link will *only* work if src and tgt are on the same resource (and
+        # even then may fail)
+        self._is_valid ()
+
+        cwdurl = saga.Url (self.url) # deep copy
+        src    = saga.Url (src_in)   # deep copy
+        tgt    = saga.Url (tgt_in)   # deep copy
+
+        rec_flag = ""
+        if  flags & saga.filesystem.RECURSIVE : 
+            raise saga.BadParameter ("'RECURSIVE' flag not  supported for link()")
+
+        if  flags & saga.filesystem.CREATE_PARENTS : 
+            self._create_parent (cwdurl, tgt)
+
+        # if src and tgt point to the same host, we just run a shell link
+        # on that host
+        if  sumisc.url_is_compatible (cwdurl, src) and \
+            sumisc.url_is_compatible (cwdurl, tgt) :
+
+            # print "shell ln"
+            ret, out, err = self.shell.run_sync ("ln -s %s %s\n" % (src.path, tgt.path))
+            if  ret != 0 :
+                raise saga.NoSuccess ("link (%s -> %s) failed (%s): (out: %s) (err: %s)" \
+                                   % (src, tgt, ret, out, err))
+
+
+        # src and tgt are on different hosts, this is not supported
+        else :
+            raise saga.BadParameter ("link is only supported on same file system as cwd")
+
+
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
@@ -576,8 +674,6 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        # if sumisc.url_is_relative (tgt) : tgt = sumisc.url_make_absolute (cwdurl, tgt)
-
         rec_flag = ""
         if  flags & saga.filesystem.RECURSIVE : 
             rec_flag  += "-r "
@@ -603,14 +699,12 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self._is_valid ()
 
         cwdurl = saga.Url (self.url) # deep copy
-        tgturl = saga.Url (tgt_in)   # deep copy
-
-        tgt_abs = sumisc.url_make_absolute (cwdurl, tgturl)
+        tgt    = saga.Url (tgt_in)   # deep copy
 
         if  flags & saga.filesystem.EXCLUSIVE : 
             # FIXME: this creates a race condition between testing for exclusive
             # mkdir and creating the dir.
-            ret, out, _ = self.shell.run_sync ("test -d %s " % tgt_abs.path)
+            ret, out, _ = self.shell.run_sync ("test -d %s " % tgt.path)
 
             if  ret != 0 :
                 raise saga.AlreadyExists ("make_dir target (%s) exists (%s)" \
@@ -622,7 +716,7 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         if  flags & saga.filesystem.CREATE_PARENTS : 
             options += "-p"
 
-        self.shell.run_sync ("mkdir %s %s" % (options, tgt_abs.path))
+        self.shell.run_sync ("mkdir %s %s" % (options, tgt.path))
 
    
     # ----------------------------------------------------------------
@@ -644,8 +738,6 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         cwdurl = saga.Url (self.url) # deep copy
         tgt    = saga.Url (tgt_in)   # deep copy
 
-        # tgt_abs = sumisc.url_make_absolute (cwdurl, tgt)
-
         ret, out, _ = self.shell.run_sync ("du -ks %s  | xargs | cut -f 1 -d ' '\n" % tgt.path)
         if  ret != 0 :
             raise saga.NoSuccess ("get size for (%s) failed (%s): (%s)" \
@@ -678,11 +770,9 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self._is_valid ()
 
         cwdurl = saga.Url (self.url) # deep copy
-        tgturl = saga.Url (tgt_in)   # deep copy
+        tgt    = saga.Url (tgt_in)   # deep copy
 
-        tgt_abs = sumisc.url_make_absolute (cwdurl, tgturl)
-
-        ret, out, _ = self.shell.run_sync ("test -d %s && test ! -h %s" % (tgt_abs.path, tgt_abs.path))
+        ret, out, _ = self.shell.run_sync ("test -d %s && test ! -h %s" % (tgt.path, tgt.path))
 
         return True if ret == 0 else False
    
@@ -705,11 +795,9 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self._is_valid ()
 
         cwdurl = saga.Url (self.url) # deep copy
-        tgturl = saga.Url (tgt_in)   # deep copy
+        tgt    = saga.Url (tgt_in)   # deep copy
 
-        tgt_abs = sumisc.url_make_absolute (cwdurl, tgturl)
-
-        ret, out, _ = self.shell.run_sync ("test -f %s && test ! -h %s" % (tgt_abs.path, tgt_abs.path))
+        ret, out, _ = self.shell.run_sync ("test -f %s && test ! -h %s" % (tgt.path, tgt.path))
 
         return True if ret == 0 else False
    
@@ -732,11 +820,9 @@ class ShellDirectory (saga.adaptors.cpi.filesystem.Directory) :
         self._is_valid ()
 
         cwdurl = saga.Url (self.url) # deep copy
-        tgturl = saga.Url (tgt_in)   # deep copy
+        tgt    = saga.Url (tgt_in)   # deep copy
 
-        tgt_abs = sumisc.url_make_absolute (cwdurl, tgturl)
-
-        ret, out, _ = self.shell.run_sync ("test -h %s" % tgt_abs.path)
+        ret, out, _ = self.shell.run_sync ("test -h %s" % tgt.path)
 
         return True if ret == 0 else False
    
@@ -769,6 +855,8 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
         _cpi_base = super  (ShellFile, self)
         _cpi_base.__init__ (api, adaptor)
 
+        self._copy_shell = None
+
 
     # ----------------------------------------------------------------
     #
@@ -814,6 +902,16 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
                                   % (tgt, cwdurl))
 
 
+    # ----------------------------------------------------------------
+    #
+    def _get_copy_shell (self, tgt) :
+
+        if  not self._copy_shell :
+            self._copy_shell = sups.PTYShell (tgt, self.session, self._logger)
+
+        return self._copy_shell
+
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
@@ -966,8 +1064,8 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
         src    = saga.Url (self.url)    # deep copy
         tgt    = saga.Url (tgt_in)      # deep copy
 
-        # if sumisc.url_is_relative (src) : src = sumisc.url_make_absolute (cwdurl, src)
-        # if sumisc.url_is_relative (tgt) : tgt = sumisc.url_make_absolute (cwdurl, tgt)
+        if sumisc.url_is_relative (src) : src = sumisc.url_make_absolute (cwdurl, src)
+        if sumisc.url_is_relative (tgt) : tgt = sumisc.url_make_absolute (cwdurl, tgt)
 
         rec_flag = ""
         if  flags & saga.filesystem.RECURSIVE : 
@@ -1032,8 +1130,8 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
                                               % (tgt))
 
                     # print "from local to remote"
-                    tmp_shell    = sups.PTYShell (tgt, self.session, self._logger)
-                    files_copied = tmp_shell.stage_to_remote (src.path, tgt.path, rec_flag)
+                    copy_shell   = self._get_copy_shell (tgt)
+                    files_copied = copy_shell.stage_to_remote (src.path, tgt.path, rec_flag)
 
                 elif sumisc.url_is_local (tgt) :
 
@@ -1043,8 +1141,8 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
                                               % (src))
 
                     # print "from remote to local"
-                    tmp_shell    = sups.PTYShell (src, self.session, self._logger)
-                    files_copied = tmp_shell.stage_from_remote (src.path, tgt.path, rec_flag)
+                    copy_shell   = self._get_copy_shell (tgt)
+                    files_copied = copy_shell.stage_from_remote (src.path, tgt.path, rec_flag)
 
                 else :
 
@@ -1053,6 +1151,44 @@ class ShellFile (saga.adaptors.cpi.filesystem.File) :
                                           % (src, tgt))
 
    
+    # ----------------------------------------------------------------
+    #
+    @SYNC_CALL
+    def link_self (self, tgt_in, flags, _from_task=None):
+
+        # link will *only* work if src and tgt are on the same resource (and
+        # even then may fail)
+        self._is_valid ()
+
+        cwdurl = saga.Url (self.url) # deep copy
+        src    = saga.Url (self.url)    # deep copy
+        tgt    = saga.Url (tgt_in)   # deep copy
+
+        rec_flag = ""
+        if  flags & saga.filesystem.RECURSIVE : 
+            raise saga.BadParameter ("'RECURSIVE' flag not  supported for link()")
+
+        if  flags & saga.filesystem.CREATE_PARENTS : 
+            self._create_parent (cwdurl, tgt)
+
+        # if src and tgt point to the same host, we just run a shell link
+        # on that host
+        if  sumisc.url_is_compatible (cwdurl, src) and \
+            sumisc.url_is_compatible (cwdurl, tgt) :
+
+            # print "shell ln"
+            ret, out, err = self.shell.run_sync ("ln -s %s %s\n" % (src.path, tgt.path))
+            if  ret != 0 :
+                raise saga.NoSuccess ("link (%s -> %s) failed (%s): (out: %s) (err: %s)" \
+                                   % (src, tgt, ret, out, err))
+
+
+        # src and tgt are on different hosts, this is not supported
+        else :
+            raise saga.BadParameter ("link is only supported on same file system as cwd")
+
+
+
     # ----------------------------------------------------------------
     #
     @SYNC_CALL
