@@ -16,6 +16,9 @@ then
   KILL_DASHES=""
 fi
 
+# we always start in the user's home dir
+\cd $HOME 2>&1 > /dev/null
+
 
 # --------------------------------------------------------------------
 #
@@ -35,6 +38,10 @@ TIMESTAMP=0
 
 PURGE_ON_START="%(PURGE_ON_START)s"
 
+# default exit value is 1, for error.  We need to set explicitly to 0 for
+# non-error conditions.
+EXIT_VAL=1
+
 # --------------------------------------------------------------------
 #
 # idle_checker is running in the background, and will terminate the wrapper
@@ -63,6 +70,7 @@ idle_checker () {
     if test -e "$BASE/quit.$ppid" 
     then
       \rm   -f  "$BASE/quit.$ppid" 
+      EXIT_VAL=0
       exit 0
     fi
 
@@ -623,6 +631,21 @@ cmd_purge () {
 }
 
 
+# --------------------------------------------------------------------
+#
+# purge tmp files for bulks etc.
+#
+cmd_purge_tmps () {
+
+  rm -f "$BASE"/bulk.*
+  rm -f "$BASE"/idle.*
+  rm -f "$BASE"/quit.*
+  find  "$BASE" -type d -mtime +30 -exec rm -rf {} \;
+  find  "$BASE" -type f -mtime +30 -exec rm -rf {} \;
+  RETVAL="purged tmp files"
+}
+
+
 # --------------------------------------------------------------------
 #
 # quit this script gracefully
@@ -633,6 +656,7 @@ cmd_quit () {
   then
     \printf "IDLE TIMEOUT\n"
     \touch "$BASE/timed_out.$$"
+    EXIT_VAL=2
   else
     \touch "$BASE/quit.$$"
   fi
@@ -648,7 +672,7 @@ cmd_quit () {
   \stty echo    >/dev/null 2>&1
   \stty echonl  >/dev/null 2>&1
 
-  exit 0
+  exit $EXIT_VAL
 }
 
 
@@ -786,9 +810,11 @@ listen() {
 \stty -echo   2> /dev/null
 \stty -echonl 2> /dev/null
 
+# FIXME: this leads to timing issues -- disable for benchmarking
 if test "$PURGE_ON_START" = "True"
 then
   cmd_purge
+  cmd_purge_tmps
 fi
 
 create_monitor
