@@ -196,15 +196,17 @@ class Directory (entry.Entry) :
     #
     @rus.takes   ('Directory', 
                   (surl.Url, basestring), 
+                  rus.optional (int),
                   rus.optional (rus.one_of (SYNC, ASYNC, TASK)))
     @rus.returns ((rus.nothing, st.Task))
-    def change_dir (self, url, ttype=None) :
+    def change_dir (self, url, flags=0, ttype=None) :
         '''
         url:           saga.Url
+        flags:         flags enum
         ttype:         saga.task.type enum
         ret:           None / saga.Task
         '''
-        return self._adaptor.change_dir (url, ttype=ttype)
+        return self._adaptor.change_dir (url, flags=flags, ttype=ttype)
   
     
     # --------------------------------------------------------------------------
@@ -351,7 +353,7 @@ class Directory (entry.Entry) :
                   rus.optional (int),
                   rus.optional (rus.one_of (SYNC, ASYNC, TASK)))
     @rus.returns ((rus.nothing, st.Task))
-    def link (self, url_1, url_2, flags=0, ttype=None) :
+    def link (self, url_1, url_2=None, flags=0, ttype=None) :
         '''
         src:           saga.Url
         tgt:           saga.Url
@@ -371,7 +373,7 @@ class Directory (entry.Entry) :
                   rus.optional (int),
                   rus.optional (rus.one_of (SYNC, ASYNC, TASK)))
     @rus.returns ((rus.nothing, st.Task))
-    def move (self, url_1, url_2, flags=0, ttype=None) :
+    def move (self, url_1, url_2=None, flags=0, ttype=None) :
         '''
         :param src: path of the entry to copy
         :param tgt: absolute URL of target directory
@@ -401,7 +403,7 @@ class Directory (entry.Entry) :
                   rus.optional (int),
                   rus.optional (rus.one_of (SYNC, ASYNC, TASK)))
     @rus.returns ((rus.nothing, st.Task))
-    def remove (self, tgt, flags=0, ttype=None) :
+    def remove (self, tgt=None, flags=0, ttype=None) :
         '''
         tgt:           saga.Url
         flags:         flags enum
@@ -411,29 +413,7 @@ class Directory (entry.Entry) :
         if tgt    :  return self._adaptor.remove (tgt, flags, ttype=ttype)
         else      :  return self._nsentry.remove (     flags, ttype=ttype)
   
-    # --------------------------------------------------------------------------
-    #
-    @rus.takes   ('Directory',
-                  rus.optional (rus.one_of (SYNC, ASYNC, TASK)))
-    @rus.returns ((rus.nothing, st.Task))
-    def remove_self (self, flags=0, ttype=None) :
-        '''
-        :param flags:  Flags to use for the operation.
 
-        ttype:         saga.task.type enum
-        ret:           None / saga.Task
-        
-        Reove the entry.
-
-        The entry is removed, and this object instance is then invalid for
-        further operations.
-
-            # remove an entry
-            entry = saga.namespace.Directory("sftp://localhost/tmp/data/data.bin")
-            entry.remove ()
-        '''
-        return self._adaptor.remove_self (flags, ttype=ttype)   
-    
     # --------------------------------------------------------------------------
     #
     @rus.takes   ('Directory', 
