@@ -249,7 +249,7 @@ class PTYShell (object) :
 
             # make sure this worked, and that we find the prompt. We use
             # a versatile prompt pattern to account for the custom shell case.
-            self.find (["^(.*[\$#%>])\s*$"])
+            self.find (["^(.*[\$#%>\]])\s*$"])
 
             # make sure this worked, and that we find the prompt. We use
             # a versatile prompt pattern to account for the custom shell case.
@@ -282,6 +282,7 @@ class PTYShell (object) :
                 
 
             self.initialized = True
+            self.finalized   = False
 
 
     # ----------------------------------------------------------------
@@ -291,7 +292,9 @@ class PTYShell (object) :
         try :
             if  kill_pty and self.pty_shell :
                 with self.pty_shell.rlock :
-                    self.pty_shell.finalize ()
+                    if not self.finalized :
+                        self.pty_shell.finalize ()
+                        self.finalized = True
 
         except Exception as e :
             pass
