@@ -72,8 +72,8 @@ _SCRIPTS = {
       # 'copy_from'     : "%(scp_env)s %(scp_exe)s   %(scp_args)s  %(s_flags)s  %(root)s/%(src)s %(tgt)s",
         'copy_to'       : "%(sftp_env)s %(sftp_exe)s %(sftp_args)s %(s_flags)s  %(host_str)s",
         'copy_from'     : "%(sftp_env)s %(sftp_exe)s %(sftp_args)s %(s_flags)s  %(host_str)s",
-        'copy_to_in'    : "mput -r %(cp_flags)s %(src)s %(tgt)s \n",
-        'copy_from_in'  : "mget -r %(cp_flags)s %(src)s %(tgt)s \n",
+        'copy_to_in'    : "mput %(cp_flags)s %(src)s %(tgt)s \n",
+        'copy_from_in'  : "mget %(cp_flags)s %(src)s %(tgt)s \n",
     },
     'sh' : { 
         'master'        : "%(sh_env)s %(sh_exe)s  %(sh_args)s",
@@ -133,9 +133,10 @@ class PTYShellFactory (object) :
     #
     def __init__ (self) :
 
-        self.logger   = rul.getLogger ('saga', 'PTYShellFactory')
-        self.registry = {}
-        self.rlock    = ru.RLock ('pty shell factory')
+        self.logger     = rul.getLogger ('saga', 'PTYShellFactory')
+        self.registry   = {}
+        self.rlock      = ru.RLock ('pty shell factory')
+        self._cp_slaves = dict()
 
 
     # --------------------------------------------------------------------------
@@ -222,15 +223,18 @@ class PTYShellFactory (object) :
                                    "Enter passphrase for .*:\s*$", # passphrase prompt
                                    "want to continue connecting",  # hostkey confirmation
                                    ".*HELLO_\\d+_SAGA$",           # prompt detection helper
-                                   "^(.*[\$#%>])\s*$"]             # greedy native shell prompt 
+                                   "^(.*[\$#%>\]])\s*$"]             # greedy native shell prompt 
 
                 # find a prompt
+                # use a very aggressive, but portable prompt setting scheme
+                pty_shell.write ("export PS1='>' >& /dev/null || set prompt='>'\n")
                 n, match = pty_shell.find (prompt_patterns, delay)
 
                 # this loop will run until we finally find the shell prompt, or
                 # if we think we have tried enough and give up.  On success
                 # we'll try to set a different prompt, and when we found that,
                 # too, we exit the loop and are be ready to running shell
+
                 # commands.
                 retries       = 0
                 retry_trigger = True
@@ -260,6 +264,8 @@ class PTYShellFactory (object) :
                         retries += 1
 
                         if  is_shell :
+                            # use a very aggressive, but portable prompt setting scheme
+                            pty_shell.write ("export PS1='>' >& /dev/null || set prompt='>'\n")
                             pty_shell.write ("printf 'HELLO_%%d_SAGA\\n' %d\n" % retries)
                             used_trigger = True
 
@@ -350,6 +356,18 @@ class PTYShellFactory (object) :
                 raise ptye.translate_exception (e)
                 
 
+    # --------------------------------------------------------------------------
+    #
+    def _get_cp_slave (self, s_cmd, info) :
+
+        host = info.get ('host_str', 'any')
+        
+        if  host not in self._cp_slaves :
+            self._cp_slaves[host] = supp.PTYProcess (s_cmd, info['logger'])
+            self._initialize_pty (self._cp_slaves[host], info)
+
+        return self._cp_slaves[host]
+
     # --------------------------------------------------------------------------
     #
     def run_shell (self, info) :
@@ -397,9 +415,7 @@ class PTYShellFactory (object) :
             s_cmd = _SCRIPTS[info['type']]['copy_to']    % repl
             s_in  = _SCRIPTS[info['type']]['copy_to_in'] % repl
 
-            cp_slave = supp.PTYProcess (s_cmd, info['logger'])
-
-            self._initialize_pty (cp_slave, info)
+            cp_slave = self._get_cp_slave (s_cmd, info)
 
             prep = ""
             if  'sftp' in s_cmd :
@@ -412,25 +428,26 @@ class PTYShellFactory (object) :
 
 
             _      = cp_slave.write    ("%s%s\n" % (prep, s_in))
-            _, out = cp_slave.find     (['[\$\>] *$'], -1)
-            _      = cp_slave.finalize ()
+            _, out = cp_slave.find     (['[\$\>\]] *$'], -1)
+            _, out = cp_slave.find     (['[\$\>\]] *$'], 1.0)
 
 
             # FIXME: we don't really get exit codes from copy
             # if  cp_slave.exit_code != 0 :
             #     raise se.NoSuccess._log (info['logger'], "file copy failed: %s" % str(out))
 
-            if  'Invalid flag' in out :
+            if 'Invalid flag' in out :
                 raise se.NoSuccess._log (info['logger'], "sftp version not supported (%s)" % str(out))
 
-            info['logger'].debug ("copy done")
-
             if 'No such file or directory' in out :
                 raise se.DoesNotExist._log (info['logger'], "file copy failed: %s" % str(out))
 
             if 'is not a directory' in out :
                 raise se.BadParameter._log (info['logger'], "File copy failed: %s" % str(out))
 
+            if 'not found' in out :
+                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+
 
             # we interpret the first word on the line as name of src file -- we
             # will return a list of those
@@ -482,9 +499,7 @@ class PTYShellFactory (object) :
             s_cmd = _SCRIPTS[info['type']]['copy_from']    % repl
             s_in  = _SCRIPTS[info['type']]['copy_from_in'] % repl
 
-            cp_slave = supp.PTYProcess (s_cmd, info['logger'])
-
-            self._initialize_pty (cp_slave, info)
+            cp_slave = self._get_cp_slave (s_cmd, info)
 
             prep = ""
             if  'sftp' in s_cmd :
@@ -500,8 +515,7 @@ class PTYShellFactory (object) :
 
 
             _      = cp_slave.write    ("%s%s\n" % (prep, s_in))
-            _, out = cp_slave.find     (['[\$\>] *$'], -1)
-            _      = cp_slave.finalize ()
+            _, out = cp_slave.find     (['[\$\>\]] *$'], -1)
 
             # FIXME: we don't really get exit codes from copy
             # if  cp_slave.exit_code != 0 :
@@ -516,6 +530,9 @@ class PTYShellFactory (object) :
             if 'is not a directory' in out :
                 raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
 
+            if 'not found' in out :
+                raise se.BadParameter._log (info['logger'], "file copy failed: %s" % out)
+
 
             # we run copy with -v, so get a list of files which have been copied
             # -- we parse that list and return it.  we interpret the *second*
@@ -655,7 +672,7 @@ class PTYShellFactory (object) :
                                     info['sftp_args'] += "-o IdentityFile=%s " % context.user_key 
 
                                     if  context.attribute_exists ("user_pass") and context.user_pass :
-                                        info['key_pass'][context.user_cert] = context.user_pass
+                                        info['key_pass'][context.user_key] = context.user_pass
 
                         if  context.type.lower () == "userpass" :
                             if  info['schema'] in _SCHEMAS_SSH + _SCHEMAS_GSI :
